﻿/* Мною, Константином aka KilkennyCat, 05 июля 2020 года создано сиё 
 * на основе "Описание формата TopoR PCB версия 1.2.0 Апрель 2017 г.".
 * k@kilkennycat.pro
 * http://kilkennycat.ru  http://kilkennycat.pro
 */

using System.Collections.Generic;
using System.Linq;
using System.Xml.Serialization;

namespace TopoR_PCB_Classes
{
    /// <summary>
    /// Раздел «Группировка объектов».
    /// </summary>
    public class Groups
    {
        /// <summary>
        /// Описание групп слоёв.
        /// </summary>
        public class LayerGroup
        {
            /// <summary>
            /// Имя объекта или ссылка на именованный объект.
            /// </summary>
            [XmlAttribute("name")]
            public string _name;

            /// <summary>
            /// Ссылка на слой или ссылка на группу слоёв.
            /// </summary>
            [
                XmlElement("LayerRef", typeof(LayerRef)),
                XmlElement("LayerGroupRef", typeof(LayerGroupRef))
            ]
            public List<object> _LayerRefs;
            public bool ShouldSerialize_LayerRefs()
            {
                return _LayerRefs?.Count > 0;
            }
            /**********************************************************************
             * Здесь находятся функции для работы с элементами класса LayerGroup. *
             * Они не являются частью формата TopoR PCB.                          *
             * *******************************************************************/
            public override string ToString()
            {
                return _name;
            }
            /*********************************************************************/
        }

        /// <summary>
        /// Описание группы цепей.
        /// </summary>
        public class NetGroup
        {
            /// <summary>
            /// Имя объекта или ссылка на именованный объект.
            /// </summary>
            [XmlAttribute("name")]
            public string _name;

            /// <summary>
            /// Ссылка на цепь или ссылка на группу цепей.
            /// </summary>
            [
                XmlElement("NetRef", typeof(NetRef)),
                XmlElement("NetGroupRef", typeof(NetGroupRef))
            ]
            public List<object> _NetRefs;
            public bool ShouldSerialize_NetRefs()
            {
                return _NetRefs?.Count > 0;
            }
        }

        /// <summary>
        /// Описание группы компонентов.
        /// </summary>
        public class CompGroup
        {
            /// <summary>
            /// Имя объекта или ссылка на именованный объект.
            /// </summary>
            [XmlAttribute("name")]
            public string _name;

            /// <summary>
            /// Ссылка на компонент на плате или ссылка на группу компонентов.
            /// </summary>
            [
                XmlElement("CompInstanceRef", typeof(CompInstanceRef)),
                XmlElement("CompGroupRef", typeof(CompGroupRef))
            ]
            public List<object> _CompRefs;
            public bool ShouldSerialize_CompRefs()
            {
                return _CompRefs?.Count > 0;
            }
        }

        /// <summary>
        /// Версия раздела.
        /// </summary>
        [XmlAttribute("version")]
        public string _version;

        /// <summary>
        /// Группы слоёв.
        /// </summary>
        [XmlArray("LayerGroups")]
        [XmlArrayItem("LayerGroup")]
        public List<LayerGroup> _LayerGroups;
        public bool ShouldSerialize_LayerGroups()
        {
            return _LayerGroups?.Count > 0;
        }
        /// <summary>
        /// Группы цепей.
        /// </summary>
        [XmlArray("NetGroups")]
        [XmlArrayItem("NetGroup")]
        public List<NetGroup> _NetGroups;
        public bool ShouldSerialize_NetGroups()
        {
            return _NetGroups?.Count > 0;
        }
        /// <summary>
        /// Группы компонентов.
        /// </summary>
        [XmlArray("CompGroups")]
        [XmlArrayItem("CompGroup")]
        public List<CompGroup> _CompGroups;
        public bool ShouldSerialize_CompGroups()
        {
            return _CompGroups?.Count > 0;
        }

        /******************************************************************
         * Здесь находятся функции для работы с элементами класса Groups. *
         * Они не являются частью формата TopoR PCB.                      *
         * ****************************************************************/

        /// <summary>
        /// Переименование ссылок на компонент, если его имя изменилось
        /// </summary>
        /// <param name="oldname">старое имя компонента</param>
        /// <param name="newname">новое имя компонента</param>
        public void Rename_compName(string oldname, string newname)
        {
            foreach (var a in _CompGroups?.Where(aa => aa._CompRefs != null))
            {
                foreach (var b in a._CompRefs.OfType<CompInstanceRef>().Where(bb => bb._ReferenceName == oldname))
                {
                    b._ReferenceName = newname;
                }
            }
        }
        /******************************************************************/
    }
}
